package service

import (
	"log"
	"reflect"

	oa "git.loafle.net/overflow/annotation-go"
	od "git.loafle.net/overflow/di-go"
	omd "git.loafle.net/overflow/model/discovery"
	"git.loafle.net/overflow_scanner/probe/discovery"
	"git.loafle.net/overflow_scanner/probe/discovery/types"
)

func init() {
	od.RegisterType(DiscoveryServiceType)
}

var DiscoveryServiceType = reflect.TypeOf((*DiscoveryService)(nil))

type DiscoveryService struct {
	oa.TypeAnnotation `annotation:"@Injectable('name': 'DiscoveryService') @Service()"`

	Discoverer discovery.Discoverer `annotation:"@Inject('name': 'Discoverer')"`

	_InitService    oa.MethodAnnotation `annotation:"@PostConstruct()"`
	_DestroyService oa.MethodAnnotation `annotation:"@PreDestroy()"`
}

func (s *DiscoveryService) InitService() {
	go func() {
		for {
			select {
			case msg, ok := <-s.Discoverer.Message():
				if !ok {
					return
				}

				switch msg.Type() {
				case types.DiscoveryMessageTypeStart:
					log.Print("Start ", msg.Data())
				case types.DiscoveryMessageTypeStop:
					log.Print("Stop ", msg.Data())
					return
				case types.DiscoveryMessageTypeQueueing:
					log.Print("Queueing ", msg.Data())
				case types.DiscoveryMessageTypeFailedQueueing:
					log.Print("FailedQueueing ", msg.Data())
				case types.DiscoveryMessageTypeTimeout:
					log.Print("Timeout ", msg.Data())
				case types.DiscoveryMessageTypeError:
					log.Print("Error ", msg.Error())
				case types.DiscoveryMessageTypeHost:
					log.Print("Host ", msg.Data())
				case types.DiscoveryMessageTypePort:
					log.Print("Port ", msg.Data())
				case types.DiscoveryMessageTypeService:
					log.Print("Service ", msg.Data())
				default:

				}
			}
		}
	}()
}

func (s *DiscoveryService) DestroyService() {

}

func (s *DiscoveryService) DiscoverHost(requesterID string, zone *omd.Zone, dh *omd.DiscoverHost) error {

	s.Discoverer.DiscoverHost(requesterID, zone, dh)
	return nil
}

func (s *DiscoveryService) DiscoverPort(requesterID string, host *omd.Host, dp *omd.DiscoverPort) error {
	s.Discoverer.DiscoverPort(requesterID, host, dp)
	return nil
}

func (s *DiscoveryService) DiscoverService(requesterID string, port *omd.Port, ds *omd.DiscoverService) error {
	s.Discoverer.DiscoverService(requesterID, port, ds)
	return nil
}

func (s *DiscoveryService) StopDiscover() error {
	return nil
}
